<?php

/**
 * Helper class for role subscription module, all methods called statically, acts as namespace
 */
class Role_subscription_form {
	/**
	 * Helper function for role subscription add and edit forms
	 * @param integer $form assoc array of form values
	 * @param string $edit_mode either 'add' or 'update'
	 * 
	 * @return mixed assoc array of form items
	 */
	function get_form($form_values = NULL, $edit_mode = 'add'){
		
		$form['title'] = array(
		    '#title' => t('Subscription Title'),
			'#type' => 'textfield',
		    '#size' => 60,
		    '#maxlength' => 255,
		    '#default_value' => $form_values['title'],
		    '#description' => 'Choose a human readable title for this subscription.',
		    '#required' => TRUE 
		);
		
		if ($edit_mode == 'add') {
			$form['rid'] = array(
			    '#type' => 'select',
			    '#title' => t('Role'),
			    '#options' => Role_subscription_db::roles_dropdown(),
			    '#default_value' => $form_values['rid'],
			    '#description' => 'Choose the role that this subscription will grant, '.l('edit roles here', 'admin/user/roles').'.',
			    '#required' => TRUE
			);
		}
		/**
		 * Can't modify role once subscription is created 
		 */
		else {
			$form['role'] = array(
			    '#type' => 'item',
			    '#title' => t('Role'),
			    '#value' => t($form_values['role']),
			    '#description' => 'This role cannot be modified.',
			);
		}
		
		if ($edit_mode == 'add') {
			$form['role_subscription_group_id'] = array(
			    '#type' => 'select',
			    '#title' => t('Subscription Group'),
			    '#options' => Role_subscription_db::groups_dropdown(),
			    '#default_value' => $form_values['role_subscription_group_id'],
			    '#description' => 'Choose subscription group for this subscription role.',
			    '#required' => TRUE
			);
		}
		/**
		 * Can't modify group once role subscription is created
		 */
		else {
			$form['group'] = array(
			    '#type' => 'item',
			    '#title' => t('Subscription Group'),
			    '#value' => $form_values['group_name'],
			    '#description' => 'This group cannot be modified.'
			);
		}
		
		$form['price'] = array(
		    '#title' => t('Subscription Price'),
			'#type' => 'textfield',
		    '#size' => 5,
		    '#maxlength' => 5,
		    '#default_value' => $form_values['price'],
		    '#description' => 'The price of the subscription after it\'s initial trial period (if any).',
		    '#required' => TRUE
		);
		
		$form['trial_amount'] = array(
		    '#title' => t('Trial Amount'),
			'#type' => 'textfield',
		    '#size' => 5,
		    '#maxlength' => 5,
		    '#default_value' => $form_values['trial_amount'],
		    '#description' => 'The price of the subscription during it\'s trial period, if applicable.'
		);
		
		$form['total_occurrences'] = array(
		    '#title' => t('Total Occurrences'),
			'#type' => 'textfield',
		    '#size' => 3,
		    '#maxlength' => 3,
		    '#default_value' => $form_values['total_occurrences'],
		    '#description' => 'Choose how many times the subscription will be billed. ' .
		    				  'Leave blank if subscription is recurring until the user cancels.'
		);
		
		$form['trial_occurrences'] = array(
		    '#title' => t('Trial Occurrences'),
			'#type' => 'textfield',
		    '#size' => 3,
		    '#maxlength' => 3,
		    '#default_value' => $form_values['trial_occurrences'],
		    '#description' => 'Choose how many times the subscription will be billed at the trial amount, if applicable. '
		);
		
		$form['description'] = array(
		    '#title' => t('Description'),
			'#type' => 'textarea',
		    '#default_value' => $form_values['description'],
		    '#description' => 'Describe the subscription. This can include business rules set up by other modules.',
		    '#required' => TRUE
		);
		
		$form['weight'] = array(
		    '#title' => t('Weight'),
			'#type' => 'select',
		    '#options' => array_combine(range(-10, 10), range(-10, 10)),
		    '#default_value' => $form_values['weight'] ? $form_values['weight'] : 0,
		    '#description' => 'Determine the order of the subscription when it is displayed. '
		);
		
		$form['submit'] = array(
			'#type' => 'submit',
			'#value' => $edit_mode == 'add' ? t('Add Role Subscription') : t('Update Role Subscription'),
		);
		
		return $form;
	}
	
	/**
	 * Common function for validating add and update subscripion forms
	 * 
	 * @return mixed
	 */
	function get_validate_form($form_id, $form_values, $edit_mode = 'add'){
		/**
		 * Title
		 */
		if (strlen($form_values['title']) > 255)
			form_set_error('title', t('The Subscription Title cannot be longer than 255 characters.'));
		
		/**
		 * Extra fields to validate if add mode
		 */
		if ($edit_mode == 'add') {
			/**
			 * User Role
			 */
			if (!ctype_digit($form_values['rid']))
				form_set_error('rid', t('The Role you submitted is invalid, the value must be an integer.'));//this should never trigger
			
			/**
			 * Subscription Group
			 */
			if (!ctype_digit($form_values['role_subscription_group_id']))
				form_set_error('role_subscription_group_id', t('The Subscription Group you submitted is invalid, the value must be an integer.'));//this should never trigger
		}	
		
		/**
		 * Subscription Price
		 */
		if (!is_numeric($form_values['price']))
			form_set_error('price', t('The Price must be a valid numeric value only.'));
			
		/**
		 * Trial Amount
		 */
		if ($form_values['trial_amount']) {
			if (!is_numeric($form_values['trial_amount']))
				form_set_error('trial_amount', t('The Trial Amount must be a valid numeric value only.'));
		}
		
		/**
		 * Total Occurrences
		 */
		if ($form_values['total_occurrences']) {
			if (!ctype_digit($form_values['total_occurrences']))
				form_set_error('total_occurrences', t('The Total Occurrences value must be a valid number greater than zero only or left blank.'));
		}
		
		/**
		 * Trial Occurrences
		 */
		if ($form_values['trial_occurrences']) {
			if (!ctype_digit($form_values['trial_occurrences']))
				form_set_error('trial_occurrences', t('The Trial Occcurrences value must be a valid number greater than zero only or left blank.'));
		}
		
		/**
		 * Weight
		 */
		if (!is_numeric($form_values['weight']))
			form_set_error('weight', t('Weight value must be a positive or negative number.'));
	}
	
	/**
	 * Return payment form items
	 */
	function get_payment_form($form_values=NULL){
		$form['first_name'] = array(
		    '#title' => t('First Name'),
			'#type' => 'textfield',
		    '#size' => 25,
		    '#maxlength' => 25,
		    '#default_value' => $form_values['first_name'],
		    '#required' => TRUE
		);
		
		$form['last_name'] = array(
		    '#title' => t('Last Name'),
			'#type' => 'textfield',
		    '#size' => 25,
		    '#maxlength' => 25,
		    '#default_value' => $form_values['last_name'],
		    '#required' => TRUE
		);
		
		$form['address'] = array(
		    '#title' => t('Billing Address'),
			'#type' => 'textfield',
		    '#size' => 25,
		    '#maxlength' => 25,
		    '#default_value' => $form_values['address'],
		    '#required' => TRUE
		);
		
		$form['city'] = array(
		    '#title' => t('City'),
			'#type' => 'textfield',
		    '#size' => 25,
		    '#maxlength' => 25,
		    '#default_value' => $form_values['city'],
		    '#required' => TRUE
		);
		
		$form['state'] = array(
		    '#type' => 'select',
		    '#title' => t('State'),
		    '#options' => array('' => '') + location_province_list_us(),//location module required, add blank
		    '#default_value' => $form_values['state'],
		    '#required' => TRUE 
		);
		
		$form['zip'] = array(
		    '#title' => t('Zip Code'),
			'#type' => 'textfield',
		    '#size' => 10,
		    '#maxlength' => 10,
		    '#default_value' => $form_values['zip'],
		    '#required' => TRUE,
		    '#description' => 'Valid format is 5 digit, e.g. xxxxx, or 5 digit plus 4, e.g. xxxxx-xxxx'
		);
		
		$form['phone'] = array(
		    '#title' => t('Phone'),
			'#type' => 'textfield',
		    '#size' => 25,
		    '#maxlength' => 25,
		    '#default_value' => $form_values['phone'],
		    '#required' => TRUE,
		    '#description' => 'Valid format is xxx-xxx-xxxx (extensions ok)'
		);
		
		$form['cc_type'] = array(
		    '#title' => t('Credit Card Type'),
			'#type' => 'select',
		    '#options' => array('' => '', 'ae' => 'American Express', 'd' => 'Discover', 'mc' => 'Mastercard', 'v' => 'Visa'),
		    '#default_value' => $form_values['cc_type'],
		    '#required' => TRUE
		);
		
		$form['cc_logos'] = array(
		    '#value' => sprintf('<img src="%s" alt="Visa Mastercard American Express Discover" width="239" height="40" />', url().drupal_get_path('module', 'role_subscription') .'/images/visa_mc_amex_disc_239x40.gif'),
		);
		
		$form['cc_number'] = array(
		    '#title' => t('Credit Card Number'),
			'#type' => 'textfield',
		    '#size' => 16,
		    '#maxlength' => 16,
		    '#default_value' => $form_values['cc_number'],
		    '#required' => TRUE,
		    '#description' => 'Typically 15 or 16 numbers, digits only, no spaces.'
		);
		
		$form['expiration_date'] = array(
		    '#title' => t('Expiration Date'),
			'#type' => 'textfield',
		    '#size' => 7,
		    '#maxlength' => 7,
		    '#default_value' => $form_values['expiration_date'],
		    '#description' => 'Give the month and year of your card\'s expiration date, e.g. mm/yyyy',
		    '#required' => TRUE
		);
		
		$form['cvv'] = array(
		    '#title' => t('CVV Number'),
			'#type' => 'textfield',
		    '#size' => 4,
		    '#maxlength' => 4,
		    '#default_value' => $form_values['cvv'],
		    '#description' => 'The CVV number is located on the back of the card and is a 3 or 4 digit number',
		    '#required' => TRUE
		);
		
		$form['submit'] = array(
			'#type' => 'submit',
			'#value' => 'Continue to Confirmation Page'
		);
		
		return $form;
	}
	
	/**
	 * Return dummy values for payment screen
	 */
	function get_dummy_payment_form_data(){
		return array(
			'first_name'	=> 'John',
			'last_name'		=> 'Doe',
			'address'		=> '1234 Rich Man Drive',
			'city'			=> 'Little Rock',
			'state'			=> 'AR',
			'zip' 			=> '72113',
			'phone'			=> '555-555-5555',
			'cc_type'		=> 'v',
			'cc_number'		=> '4111111111111111',//default fake credit card number
			'expiration_date' => date('m/Y'),
			'cvv'			=> '123'
		);
	}

	/**
	 * Validate payment form
	 */	
	function validate_payment_form($form_id, $form_values){

		/**
		 * First Name
		 */
		if (strlen($form_values['first_name']) > 255)
			form_set_error('first_name', t('The First Name field cannot be more than 255 characters long.'));
	
		/**
		 * Last Name
		 */
		if (strlen($form_values['last_name']) > 255)
			form_set_error('last_name', t('The Last Name field cannot be more than 255 characters long.'));
			
		/**
		 * Billing Address
		 */
		if (strlen($form_values['billing_address']) > 255)
			form_set_error('billing_address', t('The Billing Address field cannot be more than 255 characters long.'));
			
		/**
		 * City
		 */
		if (strlen($form_values['city']) > 255)
			form_set_error('city', t('The Billing Address field cannot be more than 255 characters long.'));
			
		/**
		 * State
		 */
		if (strlen($form_values['state']) != 2)
			form_set_error('state', t('Invalid State selected.'));//should never trigger
			
		/**
		 * ZIP Code
		 */
		if (!preg_match('/^(\d{5}-\d{4})|(\d{5})$/', $form_values['zip']))
			form_set_error('zip', t('Invalid Zip Code entered. 5 digit and 5 digit plus 4 digit zip codes are valid.'));
			
		/**
		 * Phone
		 */
		if (!preg_match('/^(\d){3}-(\d){3}-(\d){4}/', $form_values['phone']))
			form_set_error('phone', t('Invalid Phone Number entered. xxx-xxx-xxxx is correct syntax, extensions are allowed.'));
			
		/**
		 * Credit Card Type
		 */
		if (!in_array($form_values['cc_type'], array('ae', 'v', 'mc', 'd')))//validate cc symbol
			form_set_error('cc_type', t('Invalid Credit Card type submitted.'));//should never trigger
		
		/**
		 * Credit Card Number
		 */	
		if (strlen($form_values['cc_number']) > 16)
			form_set_error('cc_number', t('The Credit Card number must be 16 numbers or less.'));
		elseif(!ctype_digit($form_values['cc_number']))
			form_set_error('cc_number', t('The Credit Card number must be digits only.'));
			
		/**
		 * Expiration Date
		 */
		if (!preg_match('/^\d{1,2}\/\d{4}$/', $form_values['expiration_date']))
			form_set_error('expiration_date', t('Invalid Expiration Date. Please enter in a 2 digit month and 4 digit year.'));
			
		/**
		 * CVV
		 */
		if (!preg_match('/^\d{3,4}$/', $form_values['cvv']))
			form_set_error('cvv', t('Invalid CVV Number. Please enter in a 3 or 4 digit number'));
			
		/**
		 * Check for errors
		 */
		if (is_array(form_get_errors()))//fail
			return FALSE;
		else
			return TRUE;//success
	}
	
	/**
	 * Get HTML table from subscription information
	 */
	function get_subscription_info_html_table($sub){
		//price
		$rows [] = array('Price', '$'.$sub->price.' billed every ' . $sub->length . ' ' . 
					preg_replace('/s$/', '(s)', $sub->unit) . 
					($sub->total_occurrences ? $sub->total_occurrences . ' times.' : '.'));
		
		//trial amount
		if ($sub->trial_amount > 0)
			$rows [] = array('Trial Amount', '$'.$sub->trial_amount.' billed for 1st '.$sub->trial_occurrences.' occurrences of the billing cycle.');
		
		$header = array(t('Term'), t('Description'));
		$output .= theme('table', $header, $rows, NULL,'');
		
		return $output;
	}
	
	/**
	 * Get assoc array of form values for subscription group
	 */
	function get_group_form($form_values = NULL, $edit_mode = 'add'){
		$form['name'] = array(
		    '#title' => t('Name'),
			'#type' => 'textfield',
		    '#size' => 25,
		    '#maxlength' => 25,
		    '#default_value' => $form_values['name'],
		    '#required' => TRUE,
		    '#description' => 'Name the subscription group, e.g. "Book Club Membership".'
		);
		
		$form['description'] = array(
		    '#title' => t('Description'),
			'#type' => 'textarea',
		    '#default_value' => $form_values['description'],
		    '#description' => 'Describe the subscription group. This will be shown on the public subscription page.',
		    '#required' => TRUE
		);
		
		/**
		 * Cannot modify length and unit of groups once they have been created
		 */
		if ($edit_mode == 'add') {
			$form['length'] = array(
			    '#title' => t('Subscription Length'),
				'#type' => 'textfield',
			    '#size' => 3,
			    '#maxlength' => 3,
			    '#default_value' => $form_values['length'] ? $form_values['length'] : 1,//default to 1 if nothing selected
			    '#description' => 'Choose length of billing cycle, 1-12 for monthly billing, 1-365 for daily billing. Note this cannot ' .
			    				  'be changed once the group has been created.',
			    '#required' => TRUE
			);
			
			$form['unit'] = array(
			    '#type' => 'select',
			    '#title' => t('Unit'),
			    '#options' => array('days' => 'days', 'months' => 'months'),
			    '#default_value' => $form_values['unit'] ? $form_values['unit'] : 'months',//default to monthly if nothing selected
			    '#description' => 'Choose at what interval billing will occur, days or months. Note this cannot be changed once the ' .
			    				  'group has been created.',
			    '#required' => TRUE 
			);
		}
		else {//show uneditable values if edit mode
			$form['length_and_unit'] = array(
				'#type' => 'item',
				'#title' => t('Subscription Length and Unit'),
				'#value' => 'Billed every '.$form_values['length'] .' '.preg_replace('/s$/', '(s)', $form_values['unit']),//format nicer
				'#description' => 'Cannot be modified'
			);
		}
		
		$form['submit'] = array(
			'#type' => 'submit',
			'#value' => $edit_mode == 'add' ? t('Add Subscription Group') : t('Update Subscription Group'),
		);
		
		return $form;
	}
	
	/**
	 * Get grouped hash of role subscription data for display in radio options
	 * 
	 * @return array role subscription group names as the hash with the role subscription array being the value for each key
	 */
	function group_subscription_radio_options(){
		$result = Role_subscription_db::find_all();//get db result
		
		while ($row = db_fetch_object($result)) 
			$groups [$row->group_name][$row->id] = '<span>'.$row->title.'</span><div class="role_subscription_radio_options">'.$row->description.'</div>';
		
		return $groups;
	}
}