<?php
// $Id: user_subscription_db.inc,v 1.1 2008/05/24 23:54:09 jaykali Exp $

/**
 * @file
 * Handles the database functions for managing user subscriptions
 */

/**
 * Functions for managing user subscriptions 
 */
class User_subscription_db {
	
	/**
	 * Cancel user subscription for ID
	 * 
	 * @param integer $id user subscription ID
	 * 
	 * @todo (add suspend/activate functionality if payment engine supports it)
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function cancel_subscription($id){
		if (!$id) return;//fail
		
		return db_query('update {role_subscription_user_payments} set status = "cancelled" where id = "%d" limit 1;', $id);
	}
	
	/**
	 * Save payment data when a user subscribes and pays. This stores limited non-sensitive data ab the payment for reference, info
	 *  about the subscription, and info connecting the merchant to drupal.
	 * 
	 * @param array $payment_info hash of payment info
	 *   - "role_subscription_id" id of subscription being subscribed to
	 *   - "first_name"
	 *   - "last_name"
	 *   - "phone"
	 * 
	 * @param object $subscription info about the subscription
	 *   - "id" role subscription id
	 * @param string merchant_id the id generated by the merchant that provides a link from this database to the payment engine (merchant)
	 * @param integer invoice_id generated by helper function, a random number with a timestamp to act as a link from the payment engine
	 *        to this database.
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function save_payment($payment_info, $subscription, $merchant_id, $invoice_id){
		global $user;
		
		$query = "insert into {role_subscription_user_payments} (role_subscription_id, uid, first_name, last_name, phone, invoice_id, merchant_id) " .
			     "values ('%d', '%d', '%s', '%s', '%s', '%s', '%s');";
			   
		return db_query($query, $subscription->id, $user->uid, $payment_info['first_name'], 
								$payment_info['last_name'], $payment_info['phone'], $invoice_id, 
								$merchant_id);//will return 1 if success
	}
	
	/**
	 * Update payment info in local database
	 * 
	 * @param array $payment_info form values for payment
	 *   - "first_name"
	 *   - "last_name"
	 *   - "phone"
	 *   
	 * @param integer $id user subscription ID
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function update_payment($payment_info, $id){
		global $user;
		
		$query = "update {role_subscription_user_payments} set first_name = '%s', last_name = '%s', phone = '%s' where id = %d;";
			   
		return db_query($query, $payment_info['first_name'], $payment_info['last_name'], $payment_info['phone'], $id);//will return 1 if success
	}
	
	/**
	 * Find all user subscriptions
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */			
	function find_all(){
		$query = "select rsp.*, rs.title, u.name as username from {role_subscription_user_payments} rsp " .
				"join {role_subscription} rs on rs.id = rsp.role_subscription_id " .
				"join {users} u on u.uid = rsp.uid " .
				"where rsp.status = 'active'; ";
	
		return db_query($query);
	}
	
	/**
	 * Find all user subscriptions for a role subscription
	 * 
	 * @param integer $id role subscription ID
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function find_all_for_subscription($id){
		$query = "select rsp.*, rs.title, u.name as username from {role_subscription_user_payments} rsp " .
				"join {role_subscription} rs on rs.id = rsp.role_subscription_id " .
				"join {users} u on u.uid = rsp.uid " .
				"where rsp.status = 'active' and rs.id = %d; ";
	
		return db_query($query, $id);
	}
	
	/**
	 * Helper function to get all user subscriptions for a role subscription in an array
	 * 
	 * @param integer $id role subscription ID
	 * 
	 * @return array list of user subscriptions for role subscription
	 */
	function get_all_for_subscription($id){
		$result = self::find_all_for_subscription($id);
		
		while ($row = db_fetch_object($result))
			$subscriptions [] = $row;
			
		return $subscriptions;
	}
	
	/**
	 * Find user subscription
	 * 
	 * @param integer $id ID of user subscription
	 * 
	 * @return object user subscription data
	 */
	function find($id){
		if (!$id) return;//fail
		
		$query = "select rsp.*, r.name as role, rs.title, u.name as username, rs.role_subscription_group_id from {role_subscription_user_payments} rsp " .
				"join {role_subscription} rs on rs.id = rsp.role_subscription_id " .
				"join {users} u on u.uid = rsp.uid " .
				"join {role} r on r.rid = rs.rid " .
				"where rsp.status = 'active' and rsp.id = '%d';";
	
		return db_fetch_object(db_query($query, $id));
	}
	
	/**
	 * Find a user's subscriptions
	 * 
	 * @param integer $uid pass in user id
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function find_all_for_user($uid){
		if (!$uid) return;//fail
		
		$query = "select rsp.*, rs.title, u.name as username from {role_subscription_user_payments} rsp " .
				"join {role_subscription} rs on rs.id = rsp.role_subscription_id " .
				"join {users} u on u.uid = rsp.uid " .
				"where rsp.status = 'active' and rsp.uid = %d; ";
		
		return db_query($query, $uid);
	}
	
	/**
	 * Get all user's subscriptions
	 * 
	 * @param integer $uid pass in user ID
	 * 
	 * @return array return associative array with role subscription ID's as the associative keys and the payment information
	 *   as the value of each array item.
	 */
	function get_all_user_subscriptions($uid){
		$result = self::find_all_for_user($uid);//get db result
	
		/**
		 * Build hash
		 */
		while ($_ = db_fetch_object($result))
			$user_subscriptions [$_->role_subscription_id] = $_;//assign role subscription as id
			
		return $user_subscriptions;
	}
	
	/**
	 * Change role subscription id for a user subscription (meaning that the user has chosen to upgrade/downgrade a subscription)
	 * 
	 * @param integer $id user subscription ID
	 * @param integer $role_subscription_id role subscription ID
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function change_role_subscription($id, $role_subscription_id){
		if (!$id OR !$role_subscription_id) return;//fail
		
		$query = 'update {role_subscription_user_payments} rsp set role_subscription_id = "%d" where id = "%d";';
		
		return db_query($query, $role_subscription_id, $id);	
	}
}